package de.w665.sharepulse.rest.mappings;

import de.w665.sharepulse.model.FileUpload;
import de.w665.sharepulse.rest.ApiRestController;
import de.w665.sharepulse.service.FileSecurityService;
import de.w665.sharepulse.service.FileService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamResource;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

@Slf4j
@RestController
public class Download extends ApiRestController {

    private final FileService fileService;
    private final FileSecurityService fileSecurityService;

    @Autowired
    public Download(FileService fileService, FileSecurityService fileSecurityService) {
        this.fileService = fileService;
        this.fileSecurityService = fileSecurityService;
    }

    @GetMapping("/download")
    public ResponseEntity<Resource> download(@RequestParam String fileId,
                                            @RequestParam(value = "password", required = false) String password) throws IOException {


        FileUpload fileUpload = fileService.getFileUploadByFileId(fileId);

        fileSecurityService.verifyDownloadPermission(fileUpload, password);

        // TODO: Update download count in database (single download will not work without this)

        File file = fileService.getFileById(fileId);


        HttpHeaders headers = new HttpHeaders();
        headers.add(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"edmbass.flac\"");
        //headers.add(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + file.getName() + "\"");
        headers.add(HttpHeaders.CACHE_CONTROL, "no-cache, no-store, must-revalidate");
        headers.add(HttpHeaders.PRAGMA, "no-cache");
        headers.add(HttpHeaders.EXPIRES, "0");

        InputStreamResource resource = new InputStreamResource(new FileInputStream(file));

        return ResponseEntity.ok()
                .headers(headers)
                .contentLength(file.length())
                .contentType(MediaType.APPLICATION_OCTET_STREAM)
                .body(resource);
    }
}
